from dassco_utils.guid import create_guid, create_derivative_guid
from unittest.mock import patch
import pytest


def test_create_guid():
    mapping = {
        "institution": {
            "test_inst": 2,
        },
        "collection": {
            "test_collect": 3,
        },
        "workstation": {
            "test_work": 4,
        },
    }

    date = "2024-08-08T12:27:21+02:00"

    expected_components = [
        '7e8',
        '8',
        '08',
        '0c',
        '1b',
        '15',
        '2',
        '003',
        '04',
        '000',
        '01e240'
    ]

    # Mock randint
    with patch('random.randint', return_value=123456):
        guid = create_guid(mapping, date, "test_inst", "test_collect", "test_work")
        expected_guid = '-'.join(expected_components)
        assert guid == expected_guid


def test_create_derivative_guid():
    parent_guid = "7e8-8-08-0c-1b-15-2-003-04-000-0d4437"
    expected_derivative_guid = "7e8-8-08-0c-1b-15-2-003-04-007-0d4437-00000"

    derivative_guid = create_derivative_guid(parent_guid, 7)

    assert expected_derivative_guid == derivative_guid


def test_create_derivative_guid_already_derived():
    derived_guid = "7e8-8-08-0c-1b-15-2-003-04-007-0d4437-00000"

    with pytest.raises(ValueError):
        create_derivative_guid(derived_guid, 5)







import random


def padding(value: str, length: int) -> str:
    """
    Supplies a string with a padding of leading zeros to a specified length
    :param value: the string to be padded
    :param length: number of digits to be shown
    :return: modified string with leading zeros
    """
    return value[2:].zfill(length)


def create_guid(mapping: dict, date_str: str, institution_name: str, collection_name: str,
                workstation_name: str) -> str:
    """
    Creates a unique ID according to the following specs:
    [date created]-[institution]-[collection]-[workstation]-[random number]-[unreserved digits]
    :param mapping: input mapping to get the values of the specific institution, collection and workstation.

        Example of the structure of the mapping:

        {
            "institution": {
                "NHMD": 0,
                "AU": 1,
            },
            "collection": {
                "Vascular plants": 0,
                "Entomology": 1
            },
            "workstation": {
                "WORKHERB0001": 0,
                "WORKHERB0002": 1,
            },
        }

    :param date_str: date the image was taken
    :param institution_name: the institution where the image was taken
    :param collection_name: the collection the image is part of
    :param workstation_name: the workstation the image was taken on
    :return: a unique GUID
    """

    institution = hex(mapping["institution"][institution_name])
    collection = hex(mapping["collection"][collection_name])
    workstation = hex(mapping["workstation"][workstation_name])

    year = hex(int(date_str[:4]))
    month = hex(int(date_str[5:7]))
    day = hex(int(date_str[8:10]))
    hour = hex(int(date_str[11:13]))
    minute = hex(int(date_str[14:16]))
    second = hex(int(date_str[17:19]))
    random_number = hex(random.randint(0, 999999))
    derivative = hex(0)

    components = [
        padding(year, 3),
        padding(month, 1),
        padding(day, 2),
        padding(hour, 2),
        padding(minute, 2),
        padding(second, 2),
        padding(institution, 1),
        padding(collection, 3),
        padding(workstation, 2),
        padding(derivative, 3),
        padding(random_number, 6)
    ]
    return '-'.join(components)


def create_derivative_guid(guid: str, derivative_number: int) -> str:
    """
    Creates a GUID for derivatives from the existing GUID of the parent asset. Only works if the parent guid is not derived as well.
    :param guid: the GUID of the parent asset
    :param derivative_number: the derivative number
    :return: a unique GUID
    """
    parts = guid.split('-')

    if parts[9] != "000":
        raise ValueError("GUID indicates that the image is already a derivative")

    parts[9] = padding(hex(derivative_number), 3)

    new_guid = '-'.join(parts) + '-00000'

    return new_guid



from datetime import datetime
from pydantic import BaseModel, Field
from typing import List, Dict, Optional

class IssueModel(BaseModel):
    category: str
    name: str
    timestamp: Optional[datetime] = None
    description: Optional[str] = None
    note: Optional[str] = None
    solved: bool = False

class LegalityModel(BaseModel):
    copyright: Optional[str] = None
    license: Optional[str] = None
    credit: Optional[str] = None

class MetadataModel(BaseModel):
    asset_created_by: Optional[str] = None
    asset_deleted_by: Optional[str] = None
    asset_guid: str
    asset_pid: Optional[str] = None
    asset_subject: Optional[str] = None
    asset_updated_by: Optional[str] = None
    audited: bool = False
    audited_by: Optional[str] = None
    barcode: List[str] = []
    camera_setting_control: Optional[str] = None
    collection: str
    complete_digitiser_list: List[str] = []
    date_asset_created_ars: Optional[datetime] = None
    date_asset_deleted_ars: Optional[datetime] = None
    date_asset_finalised: Optional[datetime] = None
    date_asset_taken: Optional[datetime] = None
    date_asset_updated_ars: Optional[datetime] = None
    date_audited: Optional[datetime] = None
    date_metadata_created_ars: Optional[datetime] = None
    date_metadata_ingested: Optional[datetime] = None
    date_metadata_updated_ars: Optional[datetime] = None
    date_pushed_to_specify: Optional[datetime] = None
    digitiser: Optional[str] = None
    external_publisher: List[str] = []
    file_format: Optional[str] = None
    funding: List[str] = []
    institution: str
    issues: List[IssueModel] = []
    legality: LegalityModel = LegalityModel()
    make_public: bool = False
    metadata_created_by: Optional[str] = None
    metadata_source: Optional[str] = None
    metadata_updated_by: Optional[str] = None
    metadata_version: Optional[str] = "v3.0.2"
    mos_id: Optional[str] = None
    multi_specimen: bool = False
    parent_guid: List[str] = []
    payload_type: Optional[str] = None
    pipeline_name: str
    preparation_type: Optional[str] = None
    push_to_specify: bool = False
    restricted_access: List[str] = []
    session_id: Optional[str] = None
    specimen_pid: Optional[str] = None
    status: Optional[str] = None
    tags: Dict[str, str] = {}
    workstation_name: str



from .models import MetadataModel
import datetime


class MetadataHandler:

    def __init__(self, **kwargs):
        """
        Initialize the MetadataHandler object with the given kwargs.
        :param kwargs: metadata fields
        """
        try:
            copenhagen_tz = datetime.timezone(datetime.timedelta(hours=2))
            metadata_created_date = datetime.datetime.now(copenhagen_tz).replace(microsecond=0).isoformat()
            self.__metadata = MetadataModel(**kwargs, date_metadata_created=metadata_created_date)
        except Exception as e:
            raise e

    def metadata_to_json(self) -> str:
        """
        Returns a JSON representation of the metadata object.
        :return: JSON representation of the metadata object.
        """
        return self.__metadata.model_dump_json(indent=2)

    def metadata_to_dict(self) -> dict:
        """
        Returns a dictionary representation of the metadata object.
        :return: Dictionary representation of the metadata object.
        """
        return self.__metadata.model_dump()



from setuptools import setup, find_packages
from pathlib import Path
from dotenv import load_dotenv
import os

load_dotenv()

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

version = os.getenv('PACKAGE_VERSION')
description = 'DaSSCo Utils'

# Setting up
setup(
    name="dassco_utils",
    version=version,
    author="DaSSCo",
    author_email="dassco@ku.dk",
    description=description,
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    install_requires=['pydantic'],
    python_requires=">=3.10",
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Operating System :: OS Independent",
    ]
)



import os
import sys

project_root = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
sys.path.append(project_root)

import json
from datetime import datetime
import unittest
from dassco_utils.metadata.main import MetadataHandler
from freezegun import freeze_time

class TestMetadata(unittest.TestCase):

    #@freeze_time("2024-08-16T08:44:57+02:00")
    def test_create_json_metadata(self):

        data = {
            'asset_guid': '7e8-8-08-0c-1b-15-2-003-04-000-0d4437',
            'date_asset_taken': '2024-08-16T08:44:57+02:00',
            'collection': 'Entomology',
            'digitiser': 'John Doe',
            'file_format': 'tif',   
            'payload_type': 'image',
            'pipeline_name': 'PIPEHERB0001',
            'preparation_type': 'sheet',
            'workstation_name': 'WORKHERB0001',
            'institution': 'NHMD',
            'funding': ["DaSSCo", "DiSSCo"],
            'legality': {'copyright':"Kanon"},
            'issues': [{'category':"buffer", 'name':"crash", 'timestamp':'2024-08-16T08:44:57+02:00', 'description':"good job", 'note':"again!!", 'solved':False}]
        }

        handler = MetadataHandler(**data)

        expected_json_output = {
            "asset_created_by":None,
            "asset_deleted_by":None,
            "asset_guid":"7e8-8-08-0c-1b-15-2-003-04-000-0d4437",
            "asset_pid":None,
            "asset_subject":None,
            "asset_updated_by":None,
            "audited":False,
            "audited_by":None,
            "barcode":[],
            "camera_setting_control":None,
            "collection":"Entomology",
            "complete_digitiser_list":[],
            "date_asset_created_ars":None,
            "date_asset_deleted_ars":None,
            "date_asset_finalised":None,
            "date_asset_taken":"2024-08-16T08:44:57+02:00",
            "date_asset_updated_ars":None,
            "date_audited":None,
            "date_metadata_created_ars":None,
            "date_metadata_ingested":None,
            "date_metadata_updated_ars":None,
            "date_pushed_to_specify":None,
            "digitiser":"John Doe",
            "external_publisher":[],
            "file_format":"tif",
            "funding":["DaSSCo", "DiSSCo"],
            "institution":"NHMD",
            "issues":[{'category':"buffer", 'name':"crash", 'timestamp':'2024-08-16T08:44:57+02:00', 'description':"good job", 'note':"again!!", 'solved':False}],
            "legality":{"copyright": "Kanon", "license": None, "credit": None},
            "make_public":False,
            "metadata_created_by":None,
            "metadata_source":None,
            "metadata_updated_by":None,
            "metadata_version":"v3.0.2",
            "mos_id":None,
            "multi_specimen":False,
            "parent_guid":[],
            "payload_type":"image",
            "pipeline_name":"PIPEHERB0001",
            "preparation_type":"sheet",
            "push_to_specify":False,
            "restricted_access":[],
            "session_id":None,
            "specimen_pid":None,
            "status":None,
            "tags":{},
            "workstation_name":"WORKHERB0001"
            }
        metadata_dict = handler.metadata_to_dict()

        for key, value in metadata_dict.items():
            
            if isinstance(value, list):
                list_number = 0
                for entry in value:
                    
                    if isinstance(entry, dict):
                        for key2, value2 in entry.items():
                            
                            if isinstance(value2, datetime) and value2 is not None:
                                value2 = datetime.strftime(value2, "%Y-%m-%dT%H:%M:%S%Z")

                            self.assertEqual(value2, expected_json_output[key][list_number][key2], f"Failed: {key2}:{value2}")

                        list_number =+ 1
                    else:
                        self.assertEqual(value, expected_json_output[key], f"Failed: {key}:{value}")
                        continue   
            
            else:
                
                if isinstance(value, datetime) and value is not None:
                    value = datetime.strftime(value, "%Y-%m-%dT%H:%M:%S%Z")

                self.assertEqual(value, expected_json_output[key], f"Failed: {key}:{value}")

        metadata_json = handler.metadata_to_json()

        mdata = json.loads(metadata_json)

        for key, value in mdata.items():

            self.assertEqual(value, expected_json_output[key], f"Failed: {key}:{value}")

if __name__ == "__main__":
    unittest.main()